
///////////////////////////////////////////////////////////////////////////////////////////////////// 
///////////////////////////////////////////////////////////////////////////////////////////////////// 
//                    
//        File Name:  NPCManager.cpp 
//      Description:  Implementation of the NPC manager
//           Author:  Ashley W Matheson. 
//                    
//        Copyright:  
//                    Confidential and Proprietary Source Code For: 
//                           Yeah ... Right, Productions. 
//                        20351 83rd Ave, Langley, BC 
//                    
//        Revisions:  
//                    04/08/2002 - AWM - Original version. 
//                    
//  Copyright (c) 2002, Yeah ... Right, Productions. All rights reserved. 
//                    
///////////////////////////////////////////////////////////////////////////////////////////////////// 
///////////////////////////////////////////////////////////////////////////////////////////////////// 
#include "NPCManager.h"


// static initialization.
NPCManager* NPCManager::_instance = NULL;

// constructor/descructor pairing
NPCManager::NPCManager()
{
}

NPCManager::~NPCManager()
{
	NPCItor itor;
	for (itor = m_NPCList.begin(); itor != m_NPCList.end(); itor++)
	{
		m_NPCList.erase(itor);
		delete (*itor);
	}
}

///////////////////////////////////////////////////////////////////////////////////////////////////// 
///////////////////////////////////////////////////////////////////////////////////////////////////// 
//                    
//      Method Name:  NPCManager::Instance 
//          Summary:  static method for getting the current instance of the NPCManager
//                    
//       Parameters:  None. 
//          Returns:  NPCManager*  
//                    
//      Description:  As the only static method in this class, it accesses the _instance
//					  static member to determine the instance of this class.
//                    
//         See Also:  
//                    
//           Author:  Ashley W Matheson. 
//                    
///////////////////////////////////////////////////////////////////////////////////////////////////// 
///////////////////////////////////////////////////////////////////////////////////////////////////// 
NPCManager* NPCManager::Instance()
{	
	if (_instance == NULL)
		_instance = new NPCManager();
	return _instance;
}

///////////////////////////////////////////////////////////////////////////////////////////////////// 
///////////////////////////////////////////////////////////////////////////////////////////////////// 
//                    
//      Method Name:  NPCManager::AddNPC 
//          Summary:  Creates and adds an NPC into the system
//                    
//       Parameters:  const char* name
// 
//          Returns:  void  
//                    
//      Description:  Create an NPCObject, and add it to the internal list of NPC's.
//                    
//         See Also:  NPCObject, <list>
//                    
//           Author:  Ashley W Matheson. 
//                    
///////////////////////////////////////////////////////////////////////////////////////////////////// 
///////////////////////////////////////////////////////////////////////////////////////////////////// 
void NPCManager::AddNPC(const char* name)
{
	NPCObject* npc = new NPCObject();
	npc->SetName(name);

	m_NPCList.push_back(npc);
}

///////////////////////////////////////////////////////////////////////////////////////////////////// 
///////////////////////////////////////////////////////////////////////////////////////////////////// 
//                    
//      Method Name:  NPCManager::DeleteNPC 
//          Summary:  Delete an NPC from the internal list of NPCs
//                    
//       Parameters:  const char* name
// 
//          Returns:  void  
//                    
//      Description:  Based on the name associated with an NPC, remove it from the internal list and
//					  free up the memory allocated by it.
//                    
//         See Also:  
//                    
//           Author:  Ashley W Matheson. 
//                    
///////////////////////////////////////////////////////////////////////////////////////////////////// 
///////////////////////////////////////////////////////////////////////////////////////////////////// 
void NPCManager::DeleteNPC(const char* name)
{
	// Locate the NPC
	NPCObject* npc = NULL;
	NPCItor itor;
	for (itor = m_NPCList.begin(); itor != m_NPCList.end(); itor++)
	{
		char sourcename[64];
		(*itor)->GetName(sourcename);
		if ( 0 == strcmp(sourcename, name) )
		{
			npc = (*itor);
			break;
		}
	}

	// Delete it
	if ( npc )
	{
		m_NPCList.erase(itor);
		delete npc;
	}
}

///////////////////////////////////////////////////////////////////////////////////////////////////// 
///////////////////////////////////////////////////////////////////////////////////////////////////// 
//                    
//      Method Name:  NPCManager::Dump 
//          Summary:  Dump to the console the current contents of the internal NPC list
//                    
//       Parameters:  None. 
//          Returns:  void  
//                    
//      Description:  Iterate over the list on NPC's and display their names.
//                    
//         See Also:  
//                    
//           Author:  Ashley W Matheson. 
//                    
///////////////////////////////////////////////////////////////////////////////////////////////////// 
///////////////////////////////////////////////////////////////////////////////////////////////////// 
void NPCManager::Dump()
{
	NPCObject* npc = NULL;
	NPCItor itor;
	printf("Begin Dumping NPC Data -------------------\n");
	for (itor = m_NPCList.begin(); itor != m_NPCList.end(); itor++)
	{
		char name[64];
		(*itor)->GetName(name);

		printf("NPC Name: %s\n", name);
	}
	printf("End Dumping NPC Data -------------------\n");
}

// Binding functions

///////////////////////////////////////////////////////////////////////////////////////////////////// 
///////////////////////////////////////////////////////////////////////////////////////////////////// 
//                    
//    Function Name:  l_addNPC 
//          Summary:  bind the NPCManager::AddNPC to l_addNPC
//                    
//       Parameters:  lua_State* luaVM
// 
//          Returns:  int  
//                    
//      Description:  
//                    
//         See Also:  
//                    
//           Author:  Ashley W Matheson. 
//                    
///////////////////////////////////////////////////////////////////////////////////////////////////// 
///////////////////////////////////////////////////////////////////////////////////////////////////// 
int l_addNPC( lua_State* luaVM)
{
	theNPCManager->AddNPC( lua_tostring(luaVM, -1) );
    lua_pushnumber( luaVM, 0 );
    return 1;
}

///////////////////////////////////////////////////////////////////////////////////////////////////// 
///////////////////////////////////////////////////////////////////////////////////////////////////// 
//                    
//    Function Name:  l_deleteNPC 
//          Summary:  bind the NPCManager::DeleteNPC to l_deleteNPC
//                    
//       Parameters:  lua_State* luaVM
// 
//          Returns:  int  
//                    
//      Description:  
//                    
//         See Also:  
//                    
//           Author:  Ashley W Matheson. 
//                    
///////////////////////////////////////////////////////////////////////////////////////////////////// 
///////////////////////////////////////////////////////////////////////////////////////////////////// 
int l_deleteNPC( lua_State* luaVM)
{
	theNPCManager->DeleteNPC( lua_tostring(luaVM, -1) );
    lua_pushnumber( luaVM, 0 );
    return 1;

}
