// Ground.cpp: implementation of the CGround class.
//
//////////////////////////////////////////////////////////////////////

#include "Ground.h"
#include "DisplayDevice.h"
#include "dxutil.h"
#include "d3dutil.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CGround::CGround()
{
    m_pGroundTexture = NULL;
    m_planeGround = D3DXPLANE( 0.0f, 1.0f, 0.0f, 0.0f );;

    m_pGroundVB = NULL;
    m_pGroundIB = NULL;

	m_dwNumGroundVertices	= (GROUND_GRIDSIZE + 1) * (GROUND_GRIDSIZE + 1);
    m_dwNumGroundIndices	= (GROUND_GRIDSIZE * GROUND_GRIDSIZE) * 6;

	m_strTextureFileName	= NULL;
}

CGround::~CGround()
{
	delete m_strTextureFileName;
}

void CGround::Initialize( char* texture )
{
	m_strTextureFileName = new char[ strlen(texture) +1 ];
	strcpy(m_strTextureFileName, texture);
}

void CGround::Invalidate()
{
	SAFE_RELEASE( m_pGroundTexture );
	SAFE_RELEASE( m_pGroundVB );
	SAFE_RELEASE( m_pGroundIB );
}

void CGround::Restore()
{
	HRESULT hr;
	LPDIRECT3DDEVICE8 pd3ddevice = theDisplayDevice->GetDevice();

    hr = pd3ddevice->CreateVertexBuffer( m_dwNumGroundVertices*sizeof(COLORVERTEX),
                                           D3DUSAGE_WRITEONLY, D3DFVF_COLORVERTEX,
                                           D3DPOOL_MANAGED, &m_pGroundVB );
    if( FAILED(hr) )
	{
		OutputDebugString( "Failed to create the Gound Vertex Buffer\n" );
        return;
	}

    // Fill vertex buffer
    COLORVERTEX* pVertices;
    if( FAILED( m_pGroundVB->Lock( 0, 0, (BYTE**)&pVertices, NULL ) ) )
	{
		OutputDebugString( "Failed to lock the Gound Vertex Buffer\n" );
        return;
	}

    // Fill in vertices
    for( DWORD zz = 0; zz <= GROUND_GRIDSIZE; zz++ )
    {
        for( DWORD xx = 0; xx <= GROUND_GRIDSIZE; xx++ )
        {
            pVertices->v.x   = GROUND_WIDTH * (xx/(FLOAT)GROUND_GRIDSIZE-0.5f);
            pVertices->v.y   = 0.0f;
            pVertices->v.z   = GROUND_HEIGHT * (zz/(FLOAT)GROUND_GRIDSIZE-0.5f);
            pVertices->color = GROUND_COLOR;
            pVertices->tu    = xx*GROUND_TILE/(FLOAT)GROUND_GRIDSIZE;
            pVertices->tv    = zz*GROUND_TILE/(FLOAT)GROUND_GRIDSIZE;
            pVertices++;
        }
    }

    m_pGroundVB->Unlock();

    // Create the index buffer
    WORD* pIndices;
    hr = pd3ddevice->CreateIndexBuffer( m_dwNumGroundIndices*sizeof(WORD),
                                          D3DUSAGE_WRITEONLY,
                                          D3DFMT_INDEX16, D3DPOOL_MANAGED,
                                          &m_pGroundIB );
    if( FAILED(hr) )
	{
		OutputDebugString( "Failed to Create the Gound Index Buffer\n" );
        return;
	}

    // Fill the index buffer
    m_pGroundIB->Lock( 0, m_dwNumGroundIndices*sizeof(WORD), (BYTE**)&pIndices, 0 );
    if( FAILED(hr) )
	{
		OutputDebugString( "Failed to lock the Gound Index Buffer\n" );
        return;
	}

    // Fill in indices
    for( DWORD z = 0; z < GROUND_GRIDSIZE; z++ )
    {
        for( DWORD x = 0; x < GROUND_GRIDSIZE; x++ )
        {
            DWORD vtx = x + z * (GROUND_GRIDSIZE+1);
            *pIndices++ = (WORD)( vtx + 1 );
            *pIndices++ = (WORD)( vtx + 0 );
            *pIndices++ = (WORD)( vtx + 0 + (GROUND_GRIDSIZE+1) );
            *pIndices++ = (WORD)( vtx + 1 );
            *pIndices++ = (WORD)( vtx + 0 + (GROUND_GRIDSIZE+1) );
            *pIndices++ = (WORD)( vtx + 1 + (GROUND_GRIDSIZE+1) );
        }
    }

    m_pGroundIB->Unlock();

    // Create textures
    if( FAILED( D3DUtil_CreateTexture( pd3ddevice, m_strTextureFileName,
                                       &m_pGroundTexture ) ) )
	{
		OutputDebugString("Unable to Load the Textur\n");
        return;
	}

}

void CGround::Destroy()
{
	Invalidate();
}

void CGround::Draw()
{
	LPDIRECT3DDEVICE8 pd3ddevice = theDisplayDevice->GetDevice();

	D3DMATERIAL8 mtrl;
	D3DUtil_InitMaterial( mtrl, 1.0f, 1.0f, 1.0f );
	pd3ddevice->SetMaterial( &mtrl );
	D3DXMATRIX matTrans;

	pd3ddevice->SetTransform( D3DTS_WORLD, D3DXMatrixTranslation( &matTrans, 0.0f, 0.0f, 0.0f ));
    pd3ddevice->SetTexture( 0, m_pGroundTexture );
    pd3ddevice->SetVertexShader( D3DFVF_COLORVERTEX );
    pd3ddevice->SetStreamSource( 0, m_pGroundVB, sizeof(COLORVERTEX) );
    pd3ddevice->SetIndices( m_pGroundIB, 0L );
    pd3ddevice->DrawIndexedPrimitive( D3DPT_TRIANGLELIST, 0, m_dwNumGroundVertices, 0, m_dwNumGroundIndices/3 );

}