// ParticleSystem.cpp: implementation of the CParticleSystem class.
//
//////////////////////////////////////////////////////////////////////

#include "ParticleSystem.h"
#include "d3dutil.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CParticleSystem::CParticleSystem()
{
	m_pGeometry		= NULL;
	m_pTexture		= NULL;
	m_nPointSize	= 1.0f;
	m_strTextureName= NULL;
}

CParticleSystem::~CParticleSystem()
{
	delete m_strTextureName;
}

void CParticleSystem::Initialize( char* texture )
{

	m_strTextureName = new char[strlen(texture)+1];
	strcpy(m_strTextureName, texture);

	m_pTexture = theTextureManager->Add(m_strTextureName);
	m_pGeometry = theGeometryManager->Add("sprite");

}

void CParticleSystem::Restore( )
{
	// Load up the texture
	m_pTexture->Restore();
	m_pGeometry->Restore();
}

void CParticleSystem::Destroy()
{
	Invalidate();
	delete m_pGeometry;
	delete m_pTexture;
}

void CParticleSystem::Draw()
{
	LPDIRECT3DDEVICE8 pDevice = theDisplayDevice->GetDevice();

	D3DMATERIAL8 mtrl;
	D3DUtil_InitMaterial( mtrl, 1.0f, 1.0f, 1.0f );
	pDevice->SetMaterial( &mtrl );
	// Set the render states for using point sprites
	pDevice->SetRenderState( D3DRS_ZWRITEENABLE, FALSE );

	pDevice->SetRenderState( D3DRS_POINTSPRITEENABLE, TRUE );
	pDevice->SetRenderState( D3DRS_POINTSCALEENABLE,  TRUE );
	pDevice->SetRenderState( D3DRS_POINTSIZE,     FtoDW(0.1f) );
	pDevice->SetRenderState( D3DRS_POINTSIZE_MIN, FtoDW(1.00f) );
	pDevice->SetRenderState( D3DRS_POINTSCALE_A,  FtoDW(1.00f) );
	pDevice->SetRenderState( D3DRS_POINTSCALE_B,  FtoDW(1.00f) );
	pDevice->SetRenderState( D3DRS_POINTSCALE_C,  FtoDW(1.00f) );

	m_pTexture->Use(theDisplayDevice->GetDevice());
	pDevice->SetRenderState( D3DRS_ALPHABLENDENABLE, TRUE );
	pDevice->SetRenderState( D3DRS_SRCBLEND, D3DBLEND_SRCALPHA );
	pDevice->SetRenderState( D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA );

	D3DXMATRIX matTrans;
	// Iterate over all the known particles
	for (	ParticleListItor itor = m_listParticles.begin();
			itor != m_listParticles.end();
			++itor)
	{
		theDisplayDevice->GetDevice()->SetTransform( D3DTS_WORLD, D3DXMatrixTranslation( &matTrans,	
																						(*itor)->m_vecPosition.x,
																						(*itor)->m_vecPosition.y,
																						(*itor)->m_vecPosition.z) );
		m_pGeometry->Draw();
	}

}

CParticle* CParticleSystem::Add( D3DVECTOR position, D3DVECTOR trajectory, D3DCOLOR diffuse, float mass)
{
	CParticle*	pParticle = NULL;

	pParticle = new CParticle();

	pParticle->Initialize( &position, &trajectory, &diffuse, mass);

	m_listParticles.push_back(pParticle);

	return pParticle;
}

void CParticleSystem::Invalidate()
{
	m_pTexture->Invalidate();
	m_pGeometry->Invalidate();
}

void CParticleSystem::Process()
{
	// Iterate over all the known particles and allow them to process
	for (	ParticleListItor itor = m_listParticles.begin();
			itor != m_listParticles.end();
			++itor)
	{
		(*itor)->Process();
	}
}